/*!
 *  \~chinese
 *  @header     HYPrinterInterface.h
 *  @abstract   打印机通讯类
 *
 *  \~english
 *  @header     HYPrinterInterface.h
 *  @abstract   Printer communication
 */

#import <Foundation/Foundation.h>
#import <CoreBluetooth/CoreBluetooth.h>
#import <ExternalAccessory/ExternalAccessory.h>

typedef NS_ENUM(NSInteger, HYBluetoothState) {
    /// 未授权，请前往系统设置授权
    HYBluetoothStateUnauthorized  = 0,
    /// 蓝牙未开
    HYBluetoothStatePoweredOff  = 1,
    /// 正常
    HYBluetoothStatePoweredOn  = 2,
};

/// 数据发送相关回调协议
typedef void(^HYEmptyParameterBlock)(void);
typedef void(^HYSendProgressBlock)(float progress);
typedef void(^HYReceiveDataBlock)(NSData *data);

#pragma mark - 打印机属性类
@interface HYPrinterMessage : NSObject

@property (nonatomic, copy) NSString        *name;
@property (nonatomic, copy) NSString        *mac;
@property (nonatomic, copy) NSString        *identifier;
@property (nonatomic, copy) NSString        *version;
@property (nonatomic, copy) NSString        *battery;
@property (nonatomic, copy) NSNumber        *rssi;
@property (nonatomic, strong) NSNumber      *distance;
@property (nonatomic, strong) CBPeripheral  *peripheral;

@end

#pragma mark - 蓝牙连接协议
@protocol HYPrinterInterfaceDelegate <NSObject>

@optional
/// BLE:发现附近蓝牙设备
- (void)HYBleDidDiscoverPrints:(NSArray<HYPrinterMessage *> *)devices;
/// 连接失败
- (void)HYDidConnectFail:(NSError *)error;
/// 连接成功
- (void)HYDidConnectSuccess;
/// 断开连接
- (void)HYDidDisconnect;

@end

#pragma mark - 通讯交互类
@interface HYPrinterInterface : NSObject

@property (nonatomic, weak) id<HYPrinterInterfaceDelegate>  delegate;
@property (nonatomic, assign) BOOL                          isConnected;
@property (nonatomic, strong) HYPrinterMessage              *deviceConnected;
@property (nonatomic, copy) HYEmptyParameterBlock           sendSuccessBlock;
@property (nonatomic, copy) HYEmptyParameterBlock           sendFailBlock;
@property (nonatomic, copy) HYSendProgressBlock             sendProgressBlock;
@property (nonatomic, copy) HYReceiveDataBlock              receiveDataBlock;

/// 单例对象
+ (HYPrinterInterface *)shared;

/// 开启扫描
- (void)HYScanBluetooth;

/// 停止扫描，连接成功会自动停止扫描
- (void)HYStopScanBluetooth;

/// 连接打印机
/// @param printer 外设对象
- (void)HYConnectPrinter:(HYPrinterMessage *)printer;

/// 断开打印机
- (void)HYDisconnect;

/// 获取手机的蓝牙状态
- (HYBluetoothState)HYGetBluetoothStatus;

/// 发送数据给外设
- (void)sendDataToPeripheral:(NSData *)data;

/// 发送数据成功回调
- (void)HYSendDataSuccessCallBack:(HYEmptyParameterBlock)sendSuccessCallBack;

/// 发送数据失败回调
- (void)HYSendDataFailCallBack:(HYEmptyParameterBlock)sendFailCallBack;

/// 发送数据进度回调
- (void)HYSendDataProgressCallBack:(HYSendProgressBlock)sendProgressCallBack;

/// 接收蓝牙返回数据回调
- (void)HYReceiveDataCallBack:(HYReceiveDataBlock)receiveDataCallBack;

/// 获取SDK版本号
- (NSString *)HYGetFrameworkVersion;

/// 获取SDK打包时间
- (NSString *)HYGetFrameworkBuildTime;

@end


